# coding=utf-8

from tweb import base_handler, myweb
from tweb.error_exception import ErrException, ERROR
from tornado import gen
import json
from ..service.async_wrap import ctrl_contract


class QueryContractsHandler(base_handler.BaseHandler):
    # @myweb.authenticated
    @gen.coroutine
    def get(self, cid, **kwargs):
        ret = yield ctrl_contract.query_contracts(cid)
        return self.write({
            'list': ret
        })


class ContractHandler(base_handler.BaseHandler):
    """
    注册/注销合约
    """

    @myweb.authenticated
    @gen.coroutine
    def post(self, cid, contract_id, **kwargs):
        passport = self.request.headers.get('x-signed-passport')
        access_token = self.request.headers.get('x-access-token')

        data = json.loads(self.request.body.decode('utf-8'))
        if 'name' not in data:
            raise ErrException(ERROR.E40000, extra='not name field')
        if 'symbol' not in data:
            raise ErrException(ERROR.E40000, extra='not symbol field')
        if 'decimals' not in data:
            raise ErrException(ERROR.E40000, extra='not decimals field')
        if 'type' not in data:
            raise ErrException(ERROR.E40000, extra='not type field, value like: main, ERC20')

        ret = yield ctrl_contract.register(cid, data, passport, access_token)
        return self.write(ret)

    @myweb.authenticated
    @gen.coroutine
    def delete(self, cid, contract_id, **kwargs):
        passport = self.request.headers.get('x-signed-passport')
        access_token = self.request.headers.get('x-access-token')

        ret = yield ctrl_contract.unregister(cid, contract_id, passport, access_token)
        return self.write(ret)

    @myweb.authenticated
    @gen.coroutine
    def get(self, cid, contract_id, **kwargs):
        # user_id = self.request.headers.get('x-user-id')
        # access_token = self.request.headers.get('x-access-token')

        ret = yield ctrl_contract.read(contract_id)
        return self.write(ret)



# coding=utf-8

import webmother
from .db import mongo
from qccwallet import routes
from .service.external import ethereum
from .extra_passport import append_extra


def init(app):
    # 初始化webmother
    webmother.init(app)

    # 初始化本系统数据库
    mongo.init()

    # 增加本系统增加的权限需求
    append_extra()

    # 初始化外部区块链模块
    ethereum.init()

    # 加载路由模块
    app.load_routes(routes)



# coding:utf-8

from .. import ctrl_contract
from asyncio import get_event_loop


async def register(cid, data, *auth_args):
    args = cid, data, *auth_args
    return await get_event_loop().run_in_executor(None, ctrl_contract.register, *args)


async def unregister(cid, contract_id, *auth_args):
    args = (cid, contract_id, *auth_args)
    return await get_event_loop().run_in_executor(None, ctrl_contract.unregister, *args)


async def read(contract_id):
    args = (contract_id,)
    return await get_event_loop().run_in_executor(None, ctrl_contract.read, *args)


async def query_contracts(cid):
    args = (cid,)
    return await get_event_loop().run_in_executor(None, ctrl_contract.query_contracts, *args)



# coding=utf-8

import config
from ..extra_passport import system_name
from ..http_handler import wallet_handler as w, contract_handler as c, entrance_handler

base = '{}/{}/{}'.format(config.VER, config.PLATFORM, system_name)
base_chain = '{}/(?P<catalog>chain|catalog)/(?P<cid>\w+)'.format(base)
base_coin = '{}/(?P<cname>coin|contract)/(?P<contract_id>\w+)'.format(base)
routes = [
    # 查询组织拥有哪些资源的许可证/通行证，许可范围有多大
    (rf"/{base}/entrance/employ/passports", entrance_handler.EmployPassportsHandler),

    # 查询我的钱包账户列表
    (rf"/{base}/mine", w.MyWalletsHandler),

    # ******************************************************
    # 以下catalog/([a-f0-9]*)的含义是指在哪个区块链下，如eth，btc，eos等等，参数为注册时生成的类型ID
    # ******************************************************

    # *************************
    # 合约相关(注意：主币也是一种合约！)

    # 查询已注册合约列表
    (rf"/{base_chain}/(?P<cname>coin|contract)/list", c.QueryContractsHandler),

    # 向平台注册/注销合约（仅管理员可以使用）
    (rf"/{base_chain}/(?P<cname>coin|contract)/(?P<contract_id>\w*)", c.ContractHandler),

    # 获取交易详情
    (rf"/{base_chain}/txid/(?P<tx_hash>[a-fA-F0-9x]*)", w.TxInfoHandler),

    # 查询钱包交易列表
    (rf"/{base_chain}/address/(?P<address>[a-fA-F0-9x]*)/tx/list", w.TxListHandler),

    # *************************
    # 常用接口

    # 创建/删除新钱包账户，含合约钱包，只是对应的contract_id不同
    (rf"/{base_coin}/wallet/(?P<wallet_id>[a-f0-9]*)", w.WalletHandler),

    # 查询余额(根据钱包地址)
    (rf"/{base_coin}/address/(?P<address>[a-fA-F0-9x]*)/balance", w.BalanceHandler),

    # 执行转账交易
    (rf"/{base_coin}/address/(?P<from_address>[a-fA-F0-9x]*)/transfer", w.TransferHandler),

    # *************************
    # 高级接口

    # 合约查看（不用签名，不用矿工费，不会写入数据）
    (rf"/{base_coin}/view", w.ViewHandler),

    # 合约交易调用（需签名，会扣矿工费，会写入/改变数据）
    (rf"/{base_coin}/address/(?P<from_address>[a-fA-F0-9x]*)/transaction", w.TransactionHandler),

]
